# comic-get.pl - Utility for fetching comics from the internet
# Copyright (C) 2004-2006 Tommi Saviranta <wnd@iki.fi>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

package ComicGet::Packages;

use strict;
use warnings FATAL => 'all';

use ComicGet::Common;
use ComicGet::Config;
use ComicGet::Files;


=item get_repo_id
Get repository ID from iterator. Must be called before any next_entry.

0: Iterator
r: (id, iter)
=cut
sub get_repo_id($) {
	my $iter = shift;
	my ($t, $new_iter) = &next_entry($iter);
	my @data = split(/: /, $t);
	return ($data[1], $new_iter);
}


=item read
Reads _Packages file and returns iterator to access it.

0: Filename
r: (repo id, iter) or undef on failure
=cut
sub read($) {
	my $file = shift;
	my $data = &ComicGet::Common::read_file($file);
	if (! defined($data)) {
		return undef;
	}
	return &get_repo_id($data);
}


=item next_entry
Returns next entry from _Packages iterator.

0: Iterator
r: (entry, iter): Next entry or undef if no more entries exist.
=cut
sub next_entry($) {
	my $iter = shift;
	my ($entry, $new_iter) = split(/\n\n/, $iter, 2);
	return ($entry, $new_iter);
}


=item entry_cmp_val
See if entry has field and value like given.

0: entry
1: key
2: value
3: Require exact match (language is still ignored)
r: 1 if found, 0 if not
=cut
sub check_entry_val($$$$) {
	my ($entry, $key, $value, $exact) = @_;

	foreach $_ (split(/\n/, $entry)) {
		if (defined($exact)) {
			return 1 if (/^$key: $value$/);
		} else {
			return 1 if (/^$key: .*$value.*/);
		}
	}
	return 0;
}


=item get_entry_field
Returns field's value or undef.

0: entry
1: key
r: value
=cut
sub get_entry_field($$) {
	my ($entry, $s_key) = @_;
	foreach $_ (split(/\n/, $entry)) {
		my ($key, $val) = /(.*):\s+(.*)/;
		if ($key eq $s_key) {
			my ($nv) = $val =~ /(.*?),/;
			return defined($nv) ? $nv : $val;
		}
	}
	return undef;
}


=item get_source_by_id
Get source matching given repository identifier.

0: Id
r: Source
=cut
sub get_source_by_id($) {
	my $id = shift;
	foreach my $file (@ComicGet::Config::get_packages_files) {
		my ($t_id, $iter) = &read($file);
		if (! defined($t_id)) {
			if (-e $file) {
				print "E: Cannot read \"$file\": " .
					$ComicGet::Common::error . "\n";
			} else {
				print "E: \"$file\" broken!\n";
			}
			print "Bailing out...\n";
			exit(1);
		}
		($t_id eq $id) && return $t_id;
	}
	return undef;
}


1;
