# comic-get.pl - Utility for fetching comics from the internet
# Copyright (C) 2004-2006 Tommi Saviranta <wnd@iki.fi>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

package ComicGet::Web;

use strict;
use warnings FATAL => 'all';

use Socket;
require "sys/syscall.ph";

my $TIMEVAL_T = "LL";
my $start;
my $timeout = 15;

my $act_line;
my $recv_bytes = 0;
my $recv_files = 0;



# internals
my ($referer, $redirect, $socket, $page_uri);
my (@cookie_domain, @cookie_path, @cookie_data);
my $debug = 0;


=item set_timeout
Set connection timeout.

0: Timeout in seconds
=cut
sub set_timeout($) {
	$timeout = shift;
	$timeout = 10 if (! defined($timeout) || $timeout < 1);
}


=item set_actline
Set action line.

0: Line
=cut
sub set_actline($) {
	$act_line = shift;
}


### myconnect
#	0: host
#	1: port
#
sub myconnect {
	local $SIG{'ALRM'} = sub { print "\nW: Connection timed out.\n"; };

	if ($recv_files == 0) {
		$start = pack($TIMEVAL_T, ());
		syscall(&SYS_gettimeofday, $start, 0) != -1
				or die("gettimeofday: $!");
	}
	$recv_files++;
	my $fd;

	alarm(3);
	my $t = socket($socket, PF_INET, SOCK_STREAM, getprotobyname('tcp'));
	if ($t == 0) {
		alarm(0);
		print "\nE: Cannot create socket: $!\n";
		return;
	}
	print "Connecting $_[0]...\n" if ($debug & 2);
	connect($socket, sockaddr_in($_[1], inet_aton($_[0])));
	$fd = select($socket);
	$| = 1;
	select($fd);
	alarm(0);
} # myconnect



### close_socket
#	no parameters
#
sub close_socket {
	close($socket);
} # close_socket



### read_socket
#	no parameters
#	ret: (header, data)
#
sub read_socket {
	my ($header, $data);
	my ($size, $done, $type, $phase);
	$header = $data = $type = "";
	$phase = $done = 0;
	while (1) {
		my $rmask = "";
		vec($rmask, fileno($socket), 1) = 1;
		my ($nfound, $rrmask) = select($rmask, "", "", $timeout);
		if ($nfound == 0) {
			print "\nE: Read timed out.\n";
			last;
		}
		my $buf;
		my $nread = sysread($socket, $buf, 1024);
		last if ($nread <= 0);

		$_ = $buf;
		if ($phase == 0) {
			$header .= $_;
			($size) = /: (\d+)/ if (/^Content-Length:\s/);
			($type) = /: ([^\s]*)/ if (/^Content-Type:\s/);
		} else {
			$data .= $_;
			$done += length($_);
		}
		if ($phase == 0 && $header =~ /\n\r?\n/) {
			($header, $data) = split(/\n\r?\n/, $header, 2);
			$phase = 1;
		}

		if (defined($act_line)) {
			if (defined($size)) {
				print "$act_line [$done B/$size B]  \r";
			} else {
				print "$act_line [$done B]  \r";
			}
			flush STDOUT;
		}
	}
	$recv_bytes += length($header) + length($data);

	return ($header, $data);
} # read_socket



### write_socket
#	0: data
#
sub write_socket {
	print $socket $_[0];
	print STDERR ">> $_[0]" if ($debug & 4);
} # write_socket



### http_get
#	0: uri
#	1: port
#	2: no-track
#	ret: page data
#
sub http_get {
	my ($base) = $_[0] =~ /[^:]*:\/\/([^\/]*)/;
	my $port = defined($_[1]) ? $_[1] : 80;
	my ($header, $data);
	my $newuri;
	my $uri = $_[0];

	do {
		($header, $data) = http_get_real($uri, $port, $_[2]);
		$newuri = get_redirect($uri, $port, $header);
		if (defined($newuri)) {
			$uri = $newuri;
			print STDERR "Redirecting: $uri...\n" if ($debug & 4);
		} else {
			undef($newuri);
		}
	} while (defined($newuri));
	
	$page_uri = $uri;	# Needed in complete_uri()
	return $data;
} # http_get



### http_get_real
#	0: uri
#	1: port
#	2: no-track
#	ret: (header, data)
#
sub http_get_real {
	my ($base, $uri) = $_[0] =~ /[^:]*:\/\/([^\/]*)(.*)/;
	my ($header, $data);

	myconnect($base, $_[1]);
	
	write_socket("GET $uri HTTP/1.0\r\n");
	write_socket("Host: $base\r\n") if (! defined($_[2]));
	write_socket("Referer: http://$referer\r\n")
			if (defined($referer));
	send_cookies($base, $uri);
	write_socket("\r\n");
	
	$referer = $base . $uri if (! defined($_[2]));
	($header, $data) = read_socket();
	close_socket();

	read_cookies($header);

	return ($header, $data);
} # http_get_real



### http_post
#	0: uri
#	1: port
#	2: data
#	3: no-track
#	ret: data
#
sub http_post {
	my ($base) = $_[0] =~ /[^:]*:\/\/([^\/]*)/;
	my $port = defined($_[1]) ? $_[1] : 80;
	my ($header, $data);
	my $uri = $_[0];
	my $newuri;
	
	$page_uri = $uri;	# Needed in complete_uri()
	($header, $data) = http_post_real($uri, $port, $_[2], $_[3]);
	$newuri = get_redirect($uri, $port, $header);
	
	($header, $data) = http_get($newuri, 80) if (defined($newuri));

	return ($header, $data);
} # http_post



### http_post_real
#	0: uri
#	1: port
#	2: data
#	3: no-track
#	ret: (header, data)
#
sub http_post_real {
	my ($base, $uri) = $_[0] =~ /.*:\/\/([^\/]*)(.*)/;
	my ($header, $data);

	connect($base, $_[1]);
	
	write_socket("POST $uri HTTP/1.0\r\n");
	write_socket("Host: $base\r\n") if (! defined($_[3]));
	write_socket("Referer: http://$referer\r\n")
			if (defined($referer));
	send_cookies($base, $uri);
	write_socket("Content-Type: application/x-www-form-urlencoded\r\n");
	write_socket("Content-Length: " . length($_[2]) . "\r\n");
	write_socket("\n");
	write_socket($_[2]);

	$referer = $base . $uri if (! defined($_[3]));
	($header, $data) = read_socket();
	print STDERR "\n" if ($debug & 4); # final line-feed,
	close_socket();

	read_cookies($header);

	return ($header, $data);
} # http_post_real



### get_redirect
#	0: uri
#	1: port
#	2: data
#	ret: uri
#
sub get_redirect {
	my ($base) = $_[0] =~ /.*?:\/\/(.*?)\//;
	my $uri;

	if ($_[2] =~ /^HTTP\/\d\.\d 30[12]/) {
		$redirect++;
		die("Too many redirects!") if ($redirect == 17);
		
		($uri) = $_[2] =~ /\nLocation: (.*?)[\r\n]/;
		$uri = "$base$uri" if ($uri =~ /^\//);
		$uri = "http://$uri" if (! ($uri =~ /^http/));
	} else {
		$redirect = 0;
	}

	return $uri;
} # get_redirect



### clear_cookies
#	no parameters
#
sub clear_cookies {
	@cookie_data = ();
	@cookie_domain = ();
	@cookie_path = ();
} # clear_cookies



### read_cookies
#	0: page
#
sub read_cookies {
	my ($data, $domain, $path);
	my $page = $_[0];
	my ($foo, $bar, $back);

	while ($page =~ /Set-Cookie: /i) {
		($_, $page) = $page =~ /.*?\nSet-Cookie: (.*?)[\r?\n](.*)/s;
		($data, $_) = /(.*?)(;.*)/;
		
		$_ = " $_ ";
		$back = $_;
		($foo, $domain, $bar) = /(.*;)domain=([^;]*)(.*)$/i;
		$_ = defined($domain) ? " $foo $bar " : $back;
		$domain = "" if (! defined($domain));
		
		($foo, $path, $bar) = /(.*;)path=([^;]*)(.*)$/i;
		$path = "" if (! defined($path));

		if (defined($data) && $data =~ /=deleted$/i) {
			my $key;
			($key) = $data =~ /(.*?)=/;
			for (my $i = 0; $i <= $#cookie_data; $i++) {
				if ($cookie_data[$i] =~ /^$key/
						&& $cookie_domain[$i]
								eq $domain
						&& $cookie_path[$i]
								eq $path) {
					splice(@cookie_domain, $i, 1);
					splice(@cookie_path, $i, 1);
					splice(@cookie_data, $i, 1);
				}
			}
		} else {
			push(@cookie_domain, $domain);
			push(@cookie_path, $path);
			push(@cookie_data, $data);
		}
	}
} # read_cookies



### send_cookies
#	0: domain
#	1: path
#
sub send_cookies {
	my ($domain, $path);
	my $cookie = "";

	for (my $i = 0; $i <= $#cookie_domain; $i++) {
		$domain = $cookie_domain[$i];
		$path = $cookie_path[$i];
		
		if (($_[0] =~ /$domain$/i) && ($_[1] =~ /^$path/)) {
			$cookie .= $cookie_data[$i] . "; ";
		}
			
	}

	if ($cookie =~ /./) {
		($cookie) = $cookie =~ /(.*); /;
		write_socket("Cookie: $cookie\r\n");
	}
} # send_cookies



### complete_uri
#	0: uri
#	ret: completed uri
#
sub complete_uri {
	my $uri = $_[0];
	my ($npath);
	
	my ($domain, $path) = $referer =~ /([^\/]*)(.*)/;
	($path) = $path =~ /(.*)\// if ($path =~ /\//);

	$_ = $_[0];

	if (/^\//) {
		return "http://$domain$_";
	}
	
	if (/^http:\/\//) {
		# http://...
		if (/http:\/\/[^\.]*\//) {
			# URL is like http://foo/bar.index
			($_) = /\/\/(.*?)\//;
		} else {
			# URL is like http://www.foobar.com/...
			return $_[0];
		}
	}
	
	if (! /^\//) {
		# ../...
		while (/^\.\.\//) {
			($path) = $path =~ /(.*)\// if (defined($path));
			($_) = /...(.*)/;
		}
		$path = "" if (! defined($path));
	}

	$_ = "/$_" if (! /^\//);

	return "http://$domain$path$_";
} # complete_uri



### get_file
#	0: uri
#	ret: file
#
sub get_file {
	my $ret;
	($ret) = $_[0] =~ /.*\/(.*)/;
	return $ret;
}



### clear_referer
#
sub clear_referer {
	undef($referer);
}


=item stats
Get network statistics.

r: (secs, recv_bytes, recv_files)
=cut
sub stats() {
	return (0, 0, 0) if (! defined($start));
	my $now = pack($TIMEVAL_T, ());
	syscall(&SYS_gettimeofday(), $now, 0) != -1 or die("gettimeofday: $!");
	my @now = unpack($TIMEVAL_T, $now);
	my @start = unpack($TIMEVAL_T, $start);
	my ($s, $us) = unpack($TIMEVAL_T, $start);
	$_ /= 1_000_000 for ($now[1], $start[1]);
	my $secs = ($now[0] + $now[1]) - ($start[0] + $start[1]);
	return ($secs, $recv_bytes, $recv_files);
}


1;
